@@ -54,8 +54,8 @@ enum {
 // Define keys, defaults, minimums, and maximums for the effect parameters
 //
 //     Name          Type     Key               Def   Min      Max      Scale
-Param( Percentage,   double,  XO("Percentage"), 0.0,  -99.0,   3000.0,  1  );
-Param( UseSBSMS,     bool,    XO("SBSMS"),     false, false,   true,    1  );
+Param( Percentage,   double,  wxT("Percentage"), 0.0,  -99.0,   3000.0,  1  );
+Param( UseSBSMS,     bool,    wxT("SBSMS"),     false, false,   true,    1  );
 
 // We warp the slider to go up to 400%, but user can enter up to 3000%
 static const double kSliderMax = 100.0;          // warped above zero to actually go up to 400%
@@ -126,6 +126,11 @@ wxString EffectChangePitch::GetDescription()
    return XO("Change the pitch of a track without changing its tempo");
 }
 
+wxString EffectChangePitch::ManualPage()
+{
+   return wxT("Change_Pitch");
+}
+
 // EffectIdentInterface implementation
 
 EffectType EffectChangePitch::GetType()
@@ -188,13 +193,13 @@ bool EffectChangePitch::Process()
       proxy.mProxyEffectName = XO("High Quality Pitch Change");
       proxy.setParameters(1.0, pitchRatio);
 
-      return proxy.DoEffect(mUIParent, mProjectRate, mTracks, mFactory, &region, false);
+      return Delegate(proxy, mUIParent, &region, false);
    }
    else
 #endif
    {
       mSoundTouch = std::make_unique<SoundTouch>();
-      SetTimeWarper(std::make_unique<IdentityTimeWarper>());
+      IdentityTimeWarper warper;
       mSoundTouch->setPitchSemiTones((float)(m_dSemitonesChange));
 #ifdef USE_MIDI
       // Pitch shifting note tracks is currently only supported by SoundTouchEffect
@@ -209,7 +214,7 @@ bool EffectChangePitch::Process()
       // eliminate the next line:
       mSemitones = m_dSemitonesChange;
 #endif
-      return EffectSoundTouch::Process();
+      return EffectSoundTouch::ProcessWithTimeWarper(warper);
    }
 }
 
@@ -397,7 +402,7 @@ void EffectChangePitch::DeduceFrequencies()
 {
    // As a neat trick, attempt to get the frequency of the note at the
    // beginning of the selection.
-   SelectedTrackListOfKindIterator iter(Track::Wave, mTracks);
+   SelectedTrackListOfKindIterator iter(Track::Wave, inputTracks());
    WaveTrack *track = (WaveTrack *) iter.First();
    if (track) {
       double rate = track->GetRate();
@@ -420,22 +425,15 @@ void EffectChangePitch::DeduceFrequencies()
       auto start = track->TimeToLongSamples(t0);
 
       auto analyzeSize = windowSize * numWindows;
-      float * buffer;
-      buffer = new float[analyzeSize];
+      Floats buffer{ analyzeSize };
 
-      float * freq;
-      freq = new float[windowSize / 2];
+      Floats freq{ windowSize / 2 };
+      Floats freqa{ windowSize / 2, true };
 
-      float * freqa;
-      freqa = new float[windowSize / 2];
-
-      for(size_t j = 0; j < windowSize / 2; j++)
-         freqa[j] = 0;
-
-      track->Get((samplePtr) buffer, floatSample, start, analyzeSize);
+      track->Get((samplePtr) buffer.get(), floatSample, start, analyzeSize);
       for(unsigned i = 0; i < numWindows; i++) {
-         ComputeSpectrum(buffer + i * windowSize, windowSize,
-                         windowSize, rate, freq, true);
+         ComputeSpectrum(buffer.get() + i * windowSize, windowSize,
+                         windowSize, rate, freq.get(), true);
          for(size_t j = 0; j < windowSize / 2; j++)
             freqa[j] += freq[j];
       }
@@ -444,10 +442,6 @@ void EffectChangePitch::DeduceFrequencies()
          if (freqa[j] > freqa[argmax])
             argmax = j;
 
-      delete [] freq;
-      delete [] freqa;
-      delete [] buffer;
-
       auto lag = (windowSize / 2 - 1) - argmax;
       m_dStartFrequency = rate / lag;
    }
